@@ -10,7 +10,7 @@
 NULL
 
 #' @export
-sql_translate_env.DBIConnection <- function(con) {
+sql_translation.DBIConnection <- function(con) {
   sql_variant(
     base_scalar,
     base_agg,
@@ -18,15 +18,6 @@ sql_translate_env.DBIConnection <- function(con) {
   )
 }
 
-#' @export
-sql_subquery.DBIConnection <- function(con, from, name = unique_name(), ...) {
-  if (is.ident(from)) {
-    setNames(from, name)
-  } else {
-    build_sql("(", from, ") ", ident(name %||% unique_table_name()), con = con)
-  }
-}
-
 #' @export
 #' @rdname sql_variant
 #' @format NULL
@@ -34,6 +25,7 @@ base_scalar <- sql_translator(
   `+`    = sql_infix("+"),
   `*`    = sql_infix("*"),
   `/`    = sql_infix("/"),
+  `%/%`  = sql_not_supported("%/%"),
   `%%`   = sql_infix("%"),
   `^`    = sql_prefix("POWER", 2),
   `-`    = function(x, y = NULL) {
@@ -51,10 +43,14 @@ base_scalar <- sql_translator(
   `$`   = sql_infix(".", pad = FALSE),
   `[[`   = function(x, i) {
     i <- enexpr(i)
-    if (!is.character(i)) {
-      stop("Can only index with strings", call. = FALSE)
+    if (is.character(i)) {
+      build_sql(x, ".", ident(i))
+    } else if (is.numeric(i)) {
+      build_sql(x, "[", as.integer(i), "]")
+    } else {
+      stop("Can only index with strings and numbers", call. = FALSE)
     }
-    build_sql(x, ".", ident(i))
+
   },
   `[` = function(x, i) {
     build_sql("CASE WHEN (", i, ") THEN (", x, ") END")
@@ -181,8 +177,8 @@ base_scalar <- sql_translator(
     sql_expr(!!x %BETWEEN% !!left %AND% !!right)
   },
 
-  pmin = sql_prefix("LEAST"),
-  pmax = sql_prefix("GREATEST"),
+  pmin = sql_aggregate_n("LEAST", "pmin"),
+  pmax = sql_aggregate_n("GREATEST", "pmax"),
 
   `%>%` = `%>%`,
 
@@ -223,6 +219,7 @@ base_scalar <- sql_translator(
   paste = sql_paste(" "),
   paste0 = sql_paste(""),
   substr = sql_substr("SUBSTR"),
+  substring = sql_substr("SUBSTR"),
 
   # stringr functions
   str_length = sql_prefix("LENGTH", 1),
@@ -289,7 +286,7 @@ sql_exp <- function(a, x) {
 base_agg <- sql_translator(
   # SQL-92 aggregates
   # http://db.apache.org/derby/docs/10.7/ref/rrefsqlj33923.html
-  n          = function() sql("COUNT()"),
+  n          = function() sql("COUNT(*)"),
   mean       = sql_aggregate("AVG", "mean"),
   var        = sql_aggregate("VARIANCE", "var"),
   sum        = sql_aggregate("SUM"),
@@ -442,177 +439,3 @@ base_no_win <- sql_translator(
   str_flatten  = win_absent("STR_FLATTEN"),
   count        = win_absent("COUNT")
 )
-
-# db_ methods -------------------------------------------------------------
-
-#' @export
-db_desc.DBIConnection <- function(x) {
-  class(x)[[1]]
-}
-
-#' @export
-db_list_tables.DBIConnection <- function(con) dbListTables(con)
-
-#' @export
-db_has_table.DBIConnection <- function(con, table) dbExistsTable(con, table)
-
-#' @export
-db_data_type.DBIConnection <- function(con, fields) {
-  vapply(fields, dbDataType, dbObj = con, FUN.VALUE = character(1))
-}
-
-#' @export
-db_save_query.DBIConnection <- function(con, sql, name, temporary = TRUE,
-                                        ...) {
-  tt_sql <- build_sql(
-    "CREATE ", if (temporary) sql("TEMPORARY "),
-    "TABLE ", as.sql(name), " AS ", sql,
-    con = con
-  )
-  dbExecute(con, tt_sql, immediate = TRUE)
-  name
-}
-
-#' @export
-db_begin.DBIConnection <- function(con, ...) {
-  dbBegin(con)
-}
-
-#' @export
-db_commit.DBIConnection <- function(con, ...) dbCommit(con)
-
-#' @export
-db_rollback.DBIConnection <- function(con, ...) dbRollback(con)
-
-#' @export
-db_write_table.DBIConnection <- function(con, table, types, values, temporary = TRUE, ...) {
-  dbWriteTable(
-    con,
-    name = dbi_quote(as.sql(table), con),
-    value = values,
-    field.types = types,
-    temporary = temporary,
-    row.names = FALSE
-  )
-
-  table
-}
-
-#' @export
-db_create_table.DBIConnection <- function(con, table, types,
-                                          temporary = TRUE, ...) {
-  assert_that(is_string(table), is.character(types))
-
-  field_names <- escape(ident(names(types)), collapse = NULL, con = con)
-  fields <- sql_vector(
-    paste0(field_names, " ", types),
-    parens = TRUE,
-    collapse = ", ",
-    con = con
-  )
-  sql <- build_sql(
-    "CREATE ", if (temporary) sql("TEMPORARY "),
-    "TABLE ", as.sql(table), " ", fields,
-    con = con
-  )
-
-  dbExecute(con, sql, immediate = TRUE)
-}
-
-#' @export
-db_insert_into.DBIConnection <- function(con, table, values, ...) {
-  dbWriteTable(con, table, values, append = TRUE, row.names = FALSE)
-}
-
-#' @export
-db_create_indexes.DBIConnection <- function(con, table, indexes = NULL,
-                                            unique = FALSE, ...) {
-  if (is.null(indexes)) return()
-  assert_that(is.list(indexes))
-
-  for (index in indexes) {
-    db_create_index(con, table, index, unique = unique, ...)
-  }
-}
-
-#' @export
-db_create_index.DBIConnection <- function(con, table, columns, name = NULL,
-                                          unique = FALSE, ...) {
-  assert_that(is_string(table), is.character(columns))
-
-  name <- name %||% paste0(c(table, columns), collapse = "_")
-  fields <- escape(ident(columns), parens = TRUE, con = con)
-  sql <- build_sql(
-    "CREATE ", if (unique) sql("UNIQUE "), "INDEX ", as.sql(name),
-    " ON ", as.sql(table), " ", fields,
-    con = con)
-
-  dbExecute(con, sql)
-}
-
-#' @export
-db_drop_table.DBIConnection <- function(con, table, force = FALSE, ...) {
-  sql <- build_sql(
-    "DROP TABLE ", if (force) sql("IF EXISTS "), as.sql(table),
-    con = con
-  )
-  dbExecute(con, sql)
-}
-
-#' @export
-db_analyze.DBIConnection <- function(con, table, ...) {
-  sql <- build_sql("ANALYZE ", as.sql(table), con = con)
-  dbExecute(con, sql)
-}
-
-#' @export
-db_explain.DBIConnection <- function(con, sql, ...) {
-  exsql <- build_sql("EXPLAIN ", sql, con = con)
-  expl <- dbGetQuery(con, exsql)
-  out <- utils::capture.output(print(expl))
-
-  paste(out, collapse = "\n")
-}
-
-#' @export
-db_query_fields.DBIConnection <- function(con, sql, ...) {
-  sql <- sql_select(con, sql("*"), sql_subquery(con, sql), where = sql("0 = 1"))
-  qry <- dbSendQuery(con, sql)
-  on.exit(dbClearResult(qry))
-
-  res <- dbFetch(qry, 0)
-  names(res)
-}
-
-#' @export
-db_query_rows.DBIConnection <- function(con, sql, ...) {
-  from <- sql_subquery(con, sql, "master")
-  rows <- build_sql("SELECT COUNT(*) FROM ", from, con = con)
-
-  as.integer(dbGetQuery(con, rows)[[1]])
-}
-
-# Utility functions ------------------------------------------------------------
-
-unique_table_name <- local({
-  function() {
-    i <- getOption("dbplyr_table_num", 0) + 1
-    options(dbplyr_table_num = i)
-    sprintf("dbplyr_%03i", i)
-  }
-})
-
-res_warn_incomplete <- function(res, hint = "n = -1") {
-  if (dbHasCompleted(res)) return()
-
-  rows <- big_mark(dbGetRowCount(res))
-  warning("Only first ", rows, " results retrieved. Use ", hint, " to retrieve all.",
-    call. = FALSE)
-}
-
-
-dbi_quote <- function(x, con) UseMethod("dbi_quote")
-dbi_quote.ident_q <- function(x, con) DBI::SQL(as.character(x))
-dbi_quote.ident <- function(x, con) DBI::dbQuoteIdentifier(con, as.character(x))
-dbi_quote.character <- function(x, con) DBI::dbQuoteString(con, x)
-dbi_quote.sql <- function(x, con) DBI::SQL(as.character(x))
