@@ -1,8 +1,9 @@
 #' @importFrom dplyr show_query
 #' @export
 show_query.tbl_lazy <- function(x, ...) {
+  sql <- remote_query(x)
   cat_line("<SQL>")
-  cat_line(remote_query(x))
+  cat_line(sql)
   invisible(x)
 }
 
